<?php
/*
 * @component ZoneNationsPay
 * @copyright Copyright (C) 2023 Druskus ZoneNations https://zonenations.com
 * @license GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\Database\ParameterType;

class ZoneNationsPayModelPayout extends ZoneNationsPayModelZoneNationsPay
{
    protected $searchInFields = ['opensimid', 'paypalaccount', 'amount_iwc', 'amount_rlc'];

    public function __construct($config = array())
    {
        $config['filter_fields'] = $this->searchInFields;
        parent::__construct($config);
    }

    protected function getListQuery()
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true);

        $query->select([
            $db->quoteName('p.id'),
            $db->quoteName('p.joomlaid'),
            $db->quoteName('p.opensimid'),
            $db->quoteName('p.amount_iwc'),
            $db->quoteName('p.amount_rlc'),
            $db->quoteName('p.xchangerate'),
            $db->quoteName('p.transactionfee'),
            $db->quoteName('p.paypalaccount'),
            $db->quoteName('p.requesttime'),
            $db->quoteName('p.status'),
            $db->quoteName('p.remarks'),
            'IFNULL(' . $db->quoteName('c.symbol') . ', ' . $db->quote(Text::_('COM_ZONENATIONSPAY_UNKNOWNCURRENCY')) . ') AS symbol',
            'IFNULL(' . $db->quoteName('u.username') . ', ' . $db->quote(Text::_('COM_ZONENATIONSPAY_NOTAVAILABLE')) . ') AS joomlausername',
            'IFNULL(' . $db->quoteName('m.balance') . ', ' . $db->quote(Text::_('COM_ZONENATIONSPAY_NOTAVAILABLE')) . ') AS iwbalance'
        ]);

        $query->from($db->quoteName('#__zonenationspay_payoutrequests', 'p'))
            ->leftJoin($db->quoteName('#__zonenationspay_currencies', 'c') . ' ON ' . $db->quoteName('p.currency_rlc') . ' = ' . $db->quoteName('c.currency'))
            ->leftJoin($db->quoteName('#__users', 'u') . ' ON ' . $db->quoteName('p.joomlaid') . ' = ' . $db->quoteName('u.id'))
            ->leftJoin($db->quoteName('#__opensim_moneybalances', 'm') . ' ON ' . $db->quoteName('p.opensimid') . ' = ' . $db->quoteName('m.user'));

        // Filter search
        $search = $this->getState('filter.search');
        if (!empty($search)) {
            $search = $db->quote('%' . str_replace(' ', '%', $db->escape(trim($search), true) . '%'));
            $query->where('(' . implode(' OR ', array_map(function($field) use ($db, $search) {
                return $db->quoteName($field) . ' LIKE ' . $search;
            }, $this->searchInFields)) . ')');
        }

        // Filter payout status
        $payoutstatus = $this->getState('filter.payoutstatus');
        if ($payoutstatus !== '') {
            if ($payoutstatus === 'unsolved') {
                $query->where($db->quoteName('p.status') . ' IN (-1, 0, 1)');
            } else {
                $query->where($db->quoteName('p.status') . ' = :payoutstatus')
                    ->bind(':payoutstatus', $payoutstatus, ParameterType::INTEGER);
            }
        }

        // Filter daterange
        $daterange = $this->getState('filter.daterange');
        if (!empty($daterange)) {
            if ($daterange == 999999) {
                $query->where($db->quoteName('p.requesttime') . ' IS NOT NULL');
            } else {
                $query->where($db->quoteName('p.requesttime') . ' >= DATE_SUB(NOW(), INTERVAL :daterange DAY)')
                    ->bind(':daterange', $daterange, ParameterType::INTEGER);
            }
        }

        $query->order($db->quoteName('p.requesttime') . ' DESC');

        return $query;
    }

    protected function populateState($ordering = null, $direction = null)
    {
        $app = Factory::getApplication();

        $search = $app->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
        $this->setState('filter.search', preg_replace('/\s+/', ' ', $search));

        $payoutstatus = $app->getUserStateFromRequest($this->context . '.filter.payoutstatus', 'filter_payoutstatus', '', 'string');
        $this->setState('filter.payoutstatus', $payoutstatus);

        $daterange = $app->getUserStateFromRequest($this->context . '.filter.daterange', 'filter_daterange', '', 'string');
        $this->setState('filter.daterange', $daterange);

        parent::populateState($ordering, $direction);
    }

    public function addTransactions($items)
    {
        if (!is_array($items)) return false;

        foreach ($items as $key => $item) {
            $paypalaccounts = $this->transactionpaypal($item->joomlaid);
            $items[$key]->transactionpaypal = $paypalaccounts;

            if (empty($paypalaccounts)) {
                $items[$key]->paypalstatus = "none";
                $items[$key]->paypalaccount = HTMLHelper::_('tooltip', Text::_('COM_ZONENATIONSPAY_PAYOUT_NOTRANSACTIONFOUND'), Text::_('COM_ZONENATIONSPAY_PAYOUT_NOTRANSACTIONFOUND_TITLE'), '', $item->paypalaccount);
            } elseif (in_array($item->paypalaccount, $paypalaccounts)) {
                $items[$key]->paypalstatus = "ok";
            } else {
                $items[$key]->paypalstatus = "other";
                $items[$key]->paypalaccount = HTMLHelper::_('tooltip', Text::_('COM_ZONENATIONSPAY_PAYOUT_OTHERSFOUND'), Text::_('COM_ZONENATIONSPAY_PAYOUT_OTHERSFOUND_TITLE'), '', $item->paypalaccount);
            }

            // Payout status symbol
            $statusMap = [
                -2 => ['class' => 'expired', 'text' => 'CANCELED'],
                -1 => ['class' => 'pending', 'text' => 'PENDING'],
                0 => ['class' => 'unpublish', 'text' => 'NEW'],
                1 => ['class' => 'publish', 'text' => 'APPROVED'],
                2 => ['class' => 'icon-16-featured', 'text' => 'FINISHED']
            ];

            $status = $statusMap[$item->status] ?? ['class' => 'notdefault', 'text' => 'UNKNOWN'];
            $items[$key]->payoutsymbol = sprintf(
                '<span class="state %s" title="%s %s"><span class="text">%s</span></span>',
                $status['class'],
                Text::_('COM_ZONENATIONSPAY_PAYOUTSTATUS_' . $status['text']),
                Text::_('COM_ZONENATIONSPAY_PAYOUTSTATUS_CHANGE'),
                Text::_('COM_ZONENATIONSPAY_PAYOUTSTATUS_' . $status['text'])
            );
            $items[$key]->currentstatus = Text::_('COM_ZONENATIONSPAY_PAYOUTSTATUS_' . $status['text']);
        }

        return $items;
    }

    public function payoutSums($items)
    {
        if (!is_array($items)) return false;

        foreach ($items as $key => $item) {
            $payoutsum = $this->payoutSum($item->joomlaid);
            $items[$key]->payoutsum = $payoutsum;

            if ($item->iwbalance == Text::_('COM_ZONENATIONSPAY_NOTAVAILABLE')) {
                $items[$key]->payoutstatus = "error";
                $items[$key]->iwbalance = HTMLHelper::_('tooltip', Text::sprintf('COM_ZONENATIONSPAY_PAYOUT_NOBALANCEFOUND', $item->opensimid), Text::_('COM_ZONENATIONSPAY_PAYOUT_NOBALANCEFOUND_TITLE'), '', $item->iwbalance);
            } elseif ($item->amount_iwc > $item->iwbalance) {
                $items[$key]->payoutstatus = "error";
                $balance = "<span>" . number_format($item->iwbalance, 0, Text::_('COM_ZONENATIONSPAY_SEPERATOR_COMMA'), Text::_('COM_ZONENATIONSPAY_SEPERATOR_THOUSAND')) . "</span>";
                $items[$key]->iwbalance = HTMLHelper::_('tooltip', Text::_('COM_ZONENATIONSPAY_PAYOUT_INSUFFICIENT'), Text::_('COM_ZONENATIONSPAY_PAYOUT_INSUFFICIENT_TITLE'), '', $balance);
            } elseif ($payoutsum > $item->iwbalance) {
                $items[$key]->payoutstatus = "warning";
                $items[$key]->iwbalance = HTMLHelper::_('tooltip', Text::_('COM_ZONENATIONSPAY_PAYOUT_INSUFFICIENTWARNING'), Text::_('COM_ZONENATIONSPAY_PAYOUT_INSUFFICIENTWARNING_TITLE'), '', number_format($item->iwbalance, 0, Text::_('COM_ZONENATIONSPAY_SEPERATOR_COMMA'), Text::_('COM_ZONENATIONSPAY_SEPERATOR_THOUSAND')));
            } else {
                $items[$key]->payoutstatus = "ok";
                $items[$key]->iwbalance = number_format($item->iwbalance, 0, Text::_('COM_ZONENATIONSPAY_SEPERATOR_COMMA'), Text::_('COM_ZONENATIONSPAY_SEPERATOR_THOUSAND'));
            }
        }

        return $items;
    }

    public function payoutSum($joomlaid)
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true)
            ->select('SUM(' . $db->quoteName('amount_iwc') . ')')
            ->from($db->quoteName('#__zonenationspay_payoutrequests'))
            ->where($db->quoteName('joomlaid') . ' = :joomlaid')
            ->where($db->quoteName('status') . ' IN (-1, 0, 1)')
            ->group($db->quoteName('joomlaid'))
            ->bind(':joomlaid', $joomlaid, ParameterType::INTEGER);

        return $db->setQuery($query)->loadResult();
    }

    public function getItemFromID($items, $id)
    {
        if (!is_array($items)) return false;
        return current(array_filter($items, function($item) use ($id) {
            return $item->id == $id;
        })) ?: false;
    }

    // New method for region rent
    public function getRegionRentForPayout($payoutId)
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true)
            ->select('r.*')
            ->from($db->quoteName('#__zonenationspay_regions', 'r'))
            ->join('INNER', $db->quoteName('#__zonenationspay_payoutrequests', 'p') . ' ON ' . $db->quoteName('p.region_id') . ' = ' . $db->quoteName('r.id'))
            ->where($db->quoteName('p.id') . ' = :payoutId')
            ->bind(':payoutId', $payoutId, ParameterType::INTEGER);

        return $db->setQuery($query)->loadObject();
    }
}