<?php
/**
 * @package     ZoneNationsPay
 * @subpackage  com_zonenationspay
 *
 * @copyright   Copyright (C) 2023 Druskus ZoneNations https://zonenations.com
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// No direct access
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Installer\InstallerAdapter;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\Folder;
use Joomla\CMS\Filesystem\File;

class Com_ZoneNationsPayInstallerScript
{
    /**
     * Minimum Joomla version required to install the extension
     *
     * @var    string
     */
    protected $minimumJoomla = '5.0';

    /**
     * Minimum PHP version required to install the extension
     *
     * @var    string
     */
    protected $minimumPhp = '8.1';

    /**
     * Minimum jOpenSim version required
     *
     * @var    string
     */
    protected $minimumJOpenSim = '0.3.2.0';

    /**
     * Method to run before an install/update/uninstall method
     *
     * @param   string            $type    The type of change (install, update or discover_install)
     * @param   InstallerAdapter  $parent  The class calling this method
     *
     * @return  boolean  True on success
     */
    public function preflight($type, $parent)
    {
        // Check for minimum Joomla version
        if (!$this->checkJoomlaVersion()) {
            return false;
        }

        // Check for minimum PHP version
        if (!$this->checkPhpVersion()) {
            return false;
        }

        // Check for jOpenSim
        if (!$this->checkJOpenSim()) {
            return false;
        }

        return true;
    }

    /**
     * Method to run after an install/update/uninstall method
     *
     * @param   string            $type    The type of change (install, update or discover_install)
     * @param   InstallerAdapter  $parent  The class calling this method
     *
     * @return  boolean  True on success
     */
    public function postflight($type, $parent)
    {
        echo Text::_('COM_ZONENATIONSPAY_INSTALLERSCRIPT_POSTFLIGHT');

        if ($type === 'install') {
            $this->installSampleData($parent);
        }

        $this->createUploadDirectories();
        $this->enablePlugin();

        return true;
    }

    /**
     * Check if Joomla version meets minimum requirement
     *
     * @return  boolean  True if requirement is met
     */
    private function checkJoomlaVersion()
    {
        if (version_compare(JVERSION, $this->minimumJoomla, '<')) {
            Factory::getApplication()->enqueueMessage(
                Text::sprintf('COM_ZONENATIONSPAY_ERROR_JOOMLA_VERSION', $this->minimumJoomla),
                'error'
            );
            return false;
        }
        return true;
    }

    /**
     * Check if PHP version meets minimum requirement
     *
     * @return  boolean  True if requirement is met
     */
    private function checkPhpVersion()
    {
        if (version_compare(PHP_VERSION, $this->minimumPhp, '<')) {
            Factory::getApplication()->enqueueMessage(
                Text::sprintf('COM_ZONENATIONSPAY_ERROR_PHP_VERSION', $this->minimumPhp),
                'error'
            );
            return false;
        }
        return true;
    }

    /**
     * Check for jOpenSim and its version
     *
     * @return  boolean  True if requirement is met
     */
    private function checkJOpenSim()
    {
        $jopensim = JPATH_SITE . '/components/com_opensim';
        if (!is_dir($jopensim)) {
            Factory::getApplication()->enqueueMessage(
                Text::_('COM_ZONENATIONSPAY_ERROR_JOPENSIM_NOT_FOUND'),
                'error'
            );
            return false;
        }

        $opensimclasspath = $jopensim . '/includes/opensim.class.php';
        require_once($opensimclasspath);
        $jopensimVersion = opensim::$version;

        if (version_compare($jopensimVersion, $this->minimumJOpenSim, '<')) {
            Factory::getApplication()->enqueueMessage(
                Text::sprintf('COM_ZONENATIONSPAY_ERROR_JOPENSIM_VERSION', $this->minimumJOpenSim),
                'error'
            );
            return false;
        }

        return true;
    }

    /**
     * Method to install sample data for the component.
     *
     * @param   InstallerAdapter  $parent  Parent object calling object.
     *
     * @return void
     */
    private function installSampleData($parent)
    {
        // Add sample data installation logic here if needed
        $db = Factory::getDbo();

        // Example: Insert sample subscription plans
        $samplePlans = [
            ['Basic', 'Basic subscription plan', 9.99, 'USD', 30, 'days', 'active'],
            ['Premium', 'Premium subscription plan', 19.99, 'USD', 30, 'days', 'active'],
            ['Pro', 'Professional subscription plan', 29.99, 'USD', 30, 'days', 'active'],
        ];

        $query = $db->getQuery(true);
        $query->insert($db->quoteName('#__zonenationspay_subscription_plans'))
              ->columns($db->quoteName(['name', 'description', 'amount', 'currency', 'duration', 'duration_unit', 'status']));

        foreach ($samplePlans as $plan) {
            $query->values(implode(',', $db->quote($plan)));
        }

        $db->setQuery($query);
        $db->execute();
    }

    /**
     * Create necessary upload directories
     *
     * @return void
     */
    private function createUploadDirectories()
    {
        $dirs = [
            JPATH_ROOT . '/images/zonenationspay',
            JPATH_ROOT . '/images/zonenationspay/receipts',
        ];

        foreach ($dirs as $dir) {
            if (!Folder::exists($dir)) {
                Folder::create($dir);
            }
        }
    }

    /**
     * Enable the ZoneNationsPay plugin
     *
     * @return void
     */
    private function enablePlugin()
    {
        $db = Factory::getDbo();
        $query = $db->getQuery(true)
            ->update($db->quoteName('#__extensions'))
            ->set($db->quoteName('enabled') . ' = 1')
            ->where($db->quoteName('type') . ' = ' . $db->quote('plugin'))
            ->where($db->quoteName('element') . ' = ' . $db->quote('zonenationspay'))
            ->where($db->quoteName('folder') . ' = ' . $db->quote('system'));
        $db->setQuery($query);
        $db->execute();
    }
}
