<?php
defined('_JEXEC') or die;

use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Cache\Cache;

class ZonenationspayControllerWebhook extends BaseController
{
    protected $logCategory = 'com_zonenationspay.webhook';
    public function handle()
    {
        try {
            $this->checkCompatibility();
            $this->checkRateLimit();
            $input = Factory::getApplication()->input;
            $params = ComponentHelper::getParams('com_zonenationspay');

            // Determine which payment gateway is sending the webhook
            $userAgent = $input->server->getString('HTTP_USER_AGENT');
            $btcpaySig = $input->server->getString('HTTP_BTCPAY_SIG');

            if (strpos($userAgent, 'BTCPayServer') !== false && $btcpaySig) {
                $this->log('Received BTCPay webhook');
                $this->handleBTCPayWebhook($params);
            } else {
                $this->log('Received Stripe webhook');
                $this->handleStripeWebhook($params);
            }
        } catch (Exception $e) {
            $this->log('Error processing webhook: ' . $e->getMessage(), Log::ERROR);
            $this->notifyAdmin('Webhook Error', 'An error occurred while processing a webhook: ' . $e->getMessage());
            header('HTTP/1.1 500 Internal Server Error');
            echo json_encode(['error' => 'An error occurred processing the webhook']);
        }

        Factory::getApplication()->close();
    }

    private function handleBTCPayWebhook($params)
    {
        $input = Factory::getApplication()->input;
        $notification_data = $input->json->getRaw();
        $btcpay_sig = $input->server->getString('HTTP_BTCPAY_SIG');

        if ($btcpay_sig) {
            $reqHash = str_replace("sha256=", "", $btcpay_sig);
            $localHash = $this->getHashMac($notification_data, $params->get('btcpay_webhook_secret')); 

            if (hash_equals($reqHash, $localHash)) {
                $data = json_decode($notification_data, true);
                $this->log('BTCPay webhook data: ' . json_encode($data));

                if ($data["webhookId"] == $params->get('btcpay_webhook_id') && $data["storeId"] == $params->get('btcpay_store_id')) {
                    $db = Factory::getDbo();
                    $db->transactionStart();

                    try {
                        if (isset($data["invoiceId"]) && $data["metadata"] != null) {
                            $this->handleBTCPayInvoice($data);
                        } elseif (isset($data["pullPaymentId"]) && isset($data["payoutId"])) {
                            $this->handleBTCPayPullPayment($data);
                        }

                        $db->transactionCommit();
                    } catch (Exception $e) {
                        $db->transactionRollback();
                        throw $e;
                    }
                }
            } else {
                $this->log('Invalid BTCPay signature', Log::WARNING);
                throw new Exception('Invalid BTCPay signature');
            }
        }
    }

    private function handleStripeWebhook($params)
    {
        $stripeSecretKey = $params->get('stripe_live_mode') ? $params->get('stripe_live_secret_key') : $params->get('stripe_test_secret_key');
        $webhookSecret = $params->get('stripe_webhook_secret');

        require_once JPATH_SITE . '/components/com_zonenationspay/plugins/stripe_gateway.php';
        $stripeGateway = new ZonenationspayStripeGateway($stripeSecretKey, $webhookSecret);
        $stripeGateway->handleWebhook();
    }

    private function handleBTCPayInvoice($data)
    {
        $model = $this->getModel('Btcpay');
        $agentId = $data["metadata"]["posData"]["avatartId"];
        $secureId = $model->getSecureId($agentId);
        $amount = $data["metadata"]["posData"]["localAmount"];
        $isWebPurchase = (bool)$data["metadata"]["posData"]["isWebPurchase"];

        if ($agentId && $secureId && $amount) {
            switch ($data["type"]) {
                case "InvoiceExpired":
                case "InvoiceInvalid":
                    $this->handleExpiredInvoice($data, $agentId, $secureId, $isWebPurchase);
                    break;
                case "InvoiceProcessing":
                    $this->handleProcessingInvoice($data, $agentId, $secureId, $isWebPurchase);
                    break;
                case "InvoiceSettled":
                    if (!(bool)$data["isRedelivery"]) {
                        $this->handleSettledInvoice($data, $agentId, $secureId, $amount, $isWebPurchase);
                    }
                    break;
            }
        }
    }

    private function handleBTCPayPullPayment($data)
    {
        $model = $this->getModel('Btcpay');

        switch ($data["type"]) {
            case "PayoutCreated":
                if ($data["payoutState"] == "AwaitingApproval") {
                    $this->handleAwaitingApprovalPayout($data, $model);
                }
                break;
            case "PayoutApproved":
                if ($data["payoutState"] == "AwaitingPayment" && !(bool)$data["isRedelivery"]) {
                    $this->handleAwaitingPaymentPayout($data, $model);
                }
                break;
            case "PayoutUpdated":
                if (!(bool)$data["isRedelivery"]) {
                    switch ($data["payoutState"]) {
                        case "InProgress":
                            $this->handleInProgressPayout($data, $model);
                            break;
                        case "Completed":
                            $this->handleCompletedPayout($data, $model);
                            break;
                        case "Cancelled":
                            $this->handleCancelledPayout($data, $model);
                            break;
                    }
                }
                break;
        }
    }

    private function handleExpiredInvoice($data, $agentId, $secureId, $isWebPurchase)
    {
        $model = $this->getModel('Btcpay');
        $message = Text::sprintf('COM_ZONENATIONSPAY_INVOICE_EXPIRED', $data["invoiceId"]);
        
        if (!$isWebPurchase) {
            $model->userAlert($agentId, $message, $secureId);
        }
        
        $model->removeInvoice($agentId, $data["invoiceId"]);
        $this->log('Invoice expired: ' . $data["invoiceId"]);
    }

    private function handleProcessingInvoice($data, $agentId, $secureId, $isWebPurchase)
    {
        $model = $this->getModel('Btcpay');
        $message = Text::sprintf('COM_ZONENATIONSPAY_INVOICE_PROCESSING', $data["invoiceId"]);
        
        if (!$isWebPurchase) {
            $model->userAlert($agentId, $message, $secureId);
        }
        
        $model->updateInvoice($agentId, $data["invoiceId"], "paid");
        $this->log('Invoice processing: ' . $data["invoiceId"]);
    }

    private function handleSettledInvoice($data, $agentId, $secureId, $amount, $isWebPurchase)
    {
        $model = $this->getModel('Btcpay');
        $params = ComponentHelper::getParams('com_zonenationspay');
        $message = Text::sprintf('COM_ZONENATIONSPAY_INVOICE_SETTLED', $data["invoiceId"], $amount, $params->get('local_currency_symbol'));

        if (!$isWebPurchase) {
            $model->userAlert($agentId, $message, $secureId);
        }
        
        $description = Text::sprintf('COM_ZONENATIONSPAY_CURRENCY_BUY_DESCRIPTION', $data["invoiceId"]);
        
        if ($model->sendMoney($agentId, (int)$amount, 5010, null, null, $description)) {
            $model->updateInvoice($agentId, $data["invoiceId"], "settled");
            $this->log('Invoice settled: ' . $data["invoiceId"]);
        } else {
            $this->log('Failed to send money for settled invoice: ' . $data["invoiceId"], Log::WARNING);
        }
    }
    private function handleAwaitingApprovalPayout($data, $model)
    {
        $ppData = $model->getPullPaymentData($data["pullPaymentId"], $data["payoutId"]);
        
        if ($ppData != null && $ppData["status"] == "new") {
            $model->updatePullPayment($ppData["agentID"], $data["pullPaymentId"], $data["payoutId"], $data["payoutState"]);
            $model->moveMoney($ppData["agentID"], $model->getBankster(), (int)$ppData["localAmount"], null, null);
            $this->log('Payout awaiting approval: ' . $data["payoutId"]);
        }
    }

    private function handleAwaitingPaymentPayout($data, $model)
    {
        $ppData = $model->getPullPaymentData($data["pullPaymentId"], $data["payoutId"]);
        
        if ($ppData != null && $ppData["status"] == "AwaitingApproval") {
            $model->updatePullPayment($ppData["agentID"], $data["pullPaymentId"], $data["payoutId"], $data["payoutState"]);
            $message = Text::sprintf('COM_ZONENATIONSPAY_WITHDRAWAL_APPROVED', $data["pullPaymentId"]);
            $model->userAlert($ppData["agentID"], $message, $model->getSecureId($ppData["agentID"]));
            $this->log('Payout awaiting payment: ' . $data["payoutId"]);
        }
    }

    private function handleInProgressPayout($data, $model)
    {
        $ppData = $model->getPullPaymentData($data["pullPaymentId"], $data["payoutId"]);
        
        if ($ppData != null && $ppData["status"] == "AwaitingPayment") {
            $model->updatePullPayment($ppData["agentID"], $data["pullPaymentId"], $data["payoutId"], $data["payoutState"]);
            $message = Text::sprintf('COM_ZONENATIONSPAY_WITHDRAWAL_IN_PROGRESS', $data["pullPaymentId"]);
            $model->userAlert($ppData["agentID"], $message, $model->getSecureId($ppData["agentID"]));
            $this->log('Payout in progress: ' . $data["payoutId"]);
        }
    }

    private function handleCompletedPayout($data, $model)
    {
        $ppData = $model->getPullPaymentData($data["pullPaymentId"], $data["payoutId"]);
        
        if ($ppData != null && ($ppData["status"] == "InProgress" || $ppData["status"] == "AwaitingPayment")) {
            $model->updatePullPayment($ppData["agentID"], $data["pullPaymentId"], $data["payoutId"], $data["payoutState"]);
            $message = Text::sprintf('COM_ZONENATIONSPAY_WITHDRAWAL_COMPLETED', $data["pullPaymentId"]);
            $model->userAlert($ppData["agentID"], $message, $model->getSecureId($ppData["agentID"]));
            $model->archivePullPayment($data["pullPaymentId"]);
            $this->log('Payout completed: ' . $data["payoutId"]);
        }
    }

    private function handleCancelledPayout($data, $model)
    {
        $ppData = $model->getPullPaymentData($data["pullPaymentId"], $data["payoutId"]);
        
        if ($ppData != null && $ppData["agentID"] != null) {
            $model->removePullPayment($ppData["agentID"], $data["pullPaymentId"], $data["payoutId"]);
            $message = Text::sprintf('COM_ZONENATIONSPAY_WITHDRAWAL_CANCELLED', $data["pullPaymentId"]);
            $model->userAlert($ppData["agentID"], $message, $model->getSecureId($ppData["agentID"]));
            $model->archivePullPayment($data["pullPaymentId"]);
            $this->log('Payout cancelled: ' . $data["payoutId"]);
        }
    }

    private function getHashMac($data, $secret)
    {
        return hash_hmac('sha256', $data, $secret);
}

    private function log($message, $level = Log::INFO)
    {
        Log::addLogger(
            array(
                'text_file' => 'com_zonenationspay_webhook.php'
            ),
            Log::ALL,
            array($this->logCategory)
        );
        Log::add($message, $level, $this->logCategory);
    }

    private function notifyAdmin($subject, $body)
    {
        $mailer = Factory::getMailer();
        $config = Factory::getConfig();
        $sender = array(
            $config->get('mailfrom'),
            $config->get('fromname')
        );
        $mailer->setSender($sender);
        $mailer->addRecipient($config->get('mailfrom'));
        $mailer->setSubject($subject);
        $mailer->setBody($body);
        $mailer->Send();
    }

    private function checkRateLimit()
    {
        $cache = Factory::getCache('com_zonenationspay', '');
        $key = 'webhook_' . $_SERVER['REMOTE_ADDR'];
        $count = $cache->get($key);

        if ($count === false) {
            $cache->store(1, $key, 60); // 1 richiesta, scade dopo 60 secondi
        } elseif ($count > 10) { // Limite di 10 richieste al minuto
            throw new Exception('Rate limit exceeded');
        } else {
            $cache->store($count + 1, $key, 60);
        }
    }

    private function checkCompatibility()
    {
        if (version_compare(JVERSION, '4.0', '<')) {
            throw new RuntimeException('This version of ZoneNationsPay requires Joomla 4.0 or newer.');
        }
        if (version_compare(PHP_VERSION, '7.4', '<')) {
            throw new RuntimeException('This version of ZoneNationsPay requires PHP 7.4 or newer.');
        }
    }
}