<?php
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Language\Text;

class ZoneNationsPayHelperBtcpay
{
    protected $params;
    protected $db;

    public function __construct()
    {
        $this->params = ComponentHelper::getParams('com_zonenationspay');
        $this->db = Factory::getDbo();
    }

    public function convertToReal($amount, $isWithdrawal)
    {
        if ($amount == 0) return 0;
        $fiatAmount = $isWithdrawal
            ? bcmul($this->params->get('local_currency_withdrawal_price'), $amount, 2)
            : bcmul($this->params->get('local_currency_price'), $amount, 2);
        $url = $this->params->get('btcpay_server_url') . '/api/rates?storeId=' . $this->params->get('btcpay_store_id') . '&currencyPairs=' . $this->params->get('btcpay_payment_cryptocurrency') . '_' . $this->params->get('btcpay_fiat');
        $rateData = $this->makeApiCall($url, 'GET');
        if ($rateData && isset($rateData[0]['rate']) && !isset($rateData['error'])) {
            $rate = $rateData[0]['rate'];
            $cryptoAmount = bcdiv($fiatAmount, $rate, 8);
            return [
                'fiatAmount' => $fiatAmount,
                'cryptoAmount' => $cryptoAmount,
                'rate' => $rate
            ];
        }
        return null;
    }

    public function userAlert($agentID, $message, $secureID = null)
    {
        if (!$this->params->get('use_currency_server', true) || !$this->isValidUUID($agentID)) {
            return false;
        }
        $userInfo = $this->getUserInfo($agentID);
        if (!$userInfo) return false;
        $sessionInfo = $this->getAvatarSession($agentID);
        if (!$sessionInfo) return false;
        $secureID = $secureID ?: $sessionInfo['secureID'];
        $req = [
            'clientUUID' => $agentID,
            'clientSessionID' => $sessionInfo['sessionID'],
            'clientSecureSessionID' => $secureID,
            'Description' => $message
        ];
        $response = $this->makeXmlRpcCall($userInfo['simip'], 'UserAlert', $req);
        return $response && isset($response['success']) ? $response['success'] : false;
    }

    public function loadBTCPayURL($agentID, $invoiceUrl, $message, $type, $secureID = null)
    {
        if (!$this->params->get('use_currency_server', true) || !$this->isValidUUID($agentID)) {
            return false;
        }
        $userInfo = $this->getUserInfo($agentID);
        if (!$userInfo) return false;
        $sessionInfo = $this->getAvatarSession($agentID);
        if (!$sessionInfo) return false;
        $secureID = $secureID ?: $sessionInfo['secureID'];
        $req = [
            'clientUUID' => $agentID,
            'clientSessionID' => $sessionInfo['sessionID'],
            'clientSecureSessionID' => $secureID,
            'Description' => $message,
            'Url' => $invoiceUrl,
            'Type' => $type
        ];
        $response = $this->makeXmlRpcCall($userInfo['simip'], 'LoadBTCPayURL', $req);
        return $response && isset($response['success']) ? $response['success'] : false;
    }

    public function getBalance($agentID, $secureID = null)
    {
        if (!$this->params->get('use_currency_server', true) || !$this->isValidUUID($agentID)) {
            return -1;
        }
        $userInfo = $this->getUserInfo($agentID);
        if (!$userInfo) return -1;
        $sessionInfo = $this->getAvatarSession($agentID);
        if (!$sessionInfo) return -1;
        $secureID = $secureID ?: $sessionInfo['secureID'];
        $req = [
            'clientUUID' => $agentID,
            'clientSessionID' => $sessionInfo['sessionID'],
            'clientSecureSessionID' => $secureID
        ];
        $response = $this->makeXmlRpcCall($userInfo['simip'], 'GetBalance', $req);
        return $response && isset($response['balance']) ? (int)$response['balance'] : -1;
    }

    public function requestInvoice($agentid, $secureid, $amount, $isWebPurchase)
    {
        $header = [
            'Accept: application/json',
            'Content-Type: application/json',
            'Authorization: token ' . $this->params->get('btcpay_api_key')
        ];
        $avatarName = $this->getAvatarName($agentid);
        $invoice = [
            "currency" => $this->params->get('btcpay_fiat'),
            "amount" => $amount,
            "metadata" => [
                "itemDesc" => $amount . " " . $this->params->get('local_currency_symbol') . " Purchase.",
                "posData" => [
                    "avatarName" => $avatarName,
                    "avatartId" => $agentid,
                    "localAmount" => $amount,
                    "txType" => "5010",
                    "isWebPurchase" => $isWebPurchase
                ]
            ],
            "checkout" => [
                "defaultPaymentMethod" => $this->params->get('btcpay_payment_cryptocurrency')
            ],
            "additionalSearchTerms" => [$avatarName, $agentid]
        ];
        $url = $this->params->get('btcpay_server_url') . "/api/v1/stores/" . $this->params->get('btcpay_store_id') . "/invoices";
        $response = $this->makeApiCall($url, 'POST', json_encode($invoice), $header);
        if ($response && isset($response['id'])) {
            $notification = Text::sprintf('COM_ZONENATIONSPAY_BTCPAY_INVOICE_CREATED', $response["id"], $response["checkoutLink"]);
            if (!$isWebPurchase) {
                $this->userAlert($agentid, $notification, $secureid);
            }
            $this->addNewBtcpayInvoice($agentid, $response["id"], "new", $amount);
            return $response;
        } elseif ($response && isset($response['message'])) {
            return ["errorMessage" => Text::sprintf('COM_ZONENATIONSPAY_BTCPAY_ERROR', $response["message"])];
        }
        return $response;
    }

    public function requestWithdrawal($agentid, $secureid, $amount, $destination)
    {
        $cost = $this->convertToReal($amount, true);
        if ($cost === null) return null;
        $pullPayment = [
            "name" => Text::sprintf('COM_ZONENATIONSPAY_WITHDRAWAL_TITLE', $this->params->get('btcpay_payment_cryptocurrency'), $this->getAvatarName($agentid)),
            "description" => Text::_('COM_ZONENATIONSPAY_WITHDRAWAL_DESCRIPTION'),
            "currency" => $this->params->get('btcpay_fiat'),
            "amount" => $this->params->get('btcpay_fiat') == $this->params->get('btcpay_payment_cryptocurrency') ? $cost['cryptoAmount'] : $cost['fiatAmount'],
            "paymentMethods" => [$this->params->get('btcpay_payment_cryptocurrency')],
            "autoApproveClaims" => false,
            "expiresAt" => time() + 172800
        ];
        $pullPaymentResponse = $this->makeApiCall(
            $this->params->get('btcpay_server_url') . "/api/v1/stores/" . $this->params->get('btcpay_store_id') . "/pull-payments",
            'POST',
            json_encode($pullPayment)
        );
        if (!$pullPaymentResponse || !isset($pullPaymentResponse['id'])) {
            return ['success' => false, 'errorMessage' => Text::_('COM_ZONENATIONSPAY_WITHDRAWAL_REQUEST_FAILED')];
        }
        $claim = [
            "destination" => $destination,
            "amount" => $cost['fiatAmount'],
            "paymentMethod" => $this->params->get('btcpay_payment_cryptocurrency'),
            "metadata" => [
                "avatarName" => $this->getAvatarName($agentid),
                "avatartId" => $agentid,
                "localAmount" => $amount,
                "rate" => $cost["rate"],
                "txType" => "5013"
            ]
        ];
        $claimResponse = $this->makeApiCall(
            $this->params->get('btcpay_server_url') . "/api/v1/pull-payments/" . $pullPaymentResponse['id'] . "/payouts",
            'POST',
            json_encode($claim)
        );
        if ($claimResponse && isset($claimResponse['pullPaymentId']) && isset($claimResponse['id']) && $claimResponse['pullPaymentId'] == $pullPaymentResponse['id']) {
            $this->addNewBtcpayPullPayment($agentid, $claimResponse['pullPaymentId'], $claimResponse['id'], "new", $cost['fiatAmount'], $amount);
            $message = Text::sprintf('COM_ZONENATIONSPAY_WITHDRAWAL_SUCCESS', $claimResponse["pullPaymentId"]);
            $this->loadBTCPayURL($agentid, $this->params->get('btcpay_server_url') . "/pull-payments/" . $claimResponse["pullPaymentId"], $message, "BTCPay Withdrawal", $secureid);
            return [
                "success" => true,
                "id" => $claimResponse["pullPaymentId"],
                "link" => $this->params->get('btcpay_server_url') . "/pull-payments/" . $claimResponse["pullPaymentId"]
            ];
        } else {
            $this->archivePullPayment($pullPaymentResponse['id']);
            return ['success' => false, 'errorMessage' => Text::_('COM_ZONENATIONSPAY_CLAIM_REQUEST_FAILED')];
        }
    }

    protected function makeApiCall($url, $method = 'GET', $data = null, $headers = [])
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        if ($data) {
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        }
        if (!empty($headers)) {
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        }
        if ($this->params->get('disable_ssl_verification', 0)) {
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        }
        $response = curl_exec($ch);
        curl_close($ch);
        return json_decode($response, true);
    }

    protected function makeXmlRpcCall($serverURI, $method, $params)
    {
        $url = parse_url($serverURI);
        $port = $url['scheme'] == "https" ? 7001 : 7000;
        $server = $this->makeUrl($serverURI, $port);
        if ($server['host'] == '') return false;
        $request = xmlrpc_encode_request($method, [$params]);
        $context = stream_context_create([
            'http' => [
                'method' => "POST",
                'header' => "Content-Type: text/xml",
                'content' => $request
            ]
        ]);
        $file = file_get_contents($server['url'], false, $context);
        if ($file === false) return false;
        return xmlrpc_decode($file);
    }

    protected function isValidUUID($uuid)
    {
        return $uuid !== null && preg_match('/^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{12}$/', $uuid);
    }

    protected function getUserInfo($agentID)
    {
        $db = $this->db;
        $query = $db->getQuery(true)
            ->select('*')
            ->from($db->quoteName('#__opensim_userprofile'))
            ->where($db->quoteName('UserID') . ' = ' . $db->quote($agentID));
        $db->setQuery($query);
        $result = $db->loadAssoc();
        if ($result) {
            $query = $db->getQuery(true)
                ->select($db->quoteName('EndPoint'))
                ->from($db->quoteName('#__opensim_presence'))
                ->where($db->quoteName('UserID') . ' = ' . $db->quote($agentID))
                ->order($db->quoteName('Login') . ' DESC');
            $db->setQuery($query, 0, 1);
            $simIP = $db->loadResult();
            $result['simip'] = $simIP;
        }
        return $result;
    }

    protected function getAvatarSession($agentID)
    {
        $db = $this->db;
        $query = $db->getQuery(true)
            ->select('*')
            ->from($db->quoteName('#__opensim_presence'))
            ->where($db->quoteName('UserID') . ' = ' . $db->quote($agentID))
            ->order($db->quoteName('Login') . ' DESC');
        $db->setQuery($query, 0, 1);
        $result = $db->loadAssoc();
        if ($result) {
            return [
                'sessionID' => $result['SessionID'],
                'secureID' => $result['SecureSessionID']
            ];
        }
        return null;
    }

    protected function getAvatarName($agentID)
    {
        $db = $this->db;
        $query = $db->getQuery(true)
            ->select($db->quoteName(['FirstName', 'LastName']))
            ->from($db->quoteName('#__opensim_useraccounts'))
            ->where($db->quoteName('PrincipalID') . ' = ' . $db->quote($agentID));
        $db->setQuery($query);
        $result = $db->loadAssoc();
        if ($result) {
            return $result['FirstName'] . ' ' . $result['LastName'];
        }
        return '';
    }

    protected function addNewBtcpayInvoice($agentid, $invoiceId, $status, $amount)
    {
        $db = $this->db;
        $query = $db->getQuery(true);
        $columns = array('agentid', 'invoiceid', 'status', 'amount', 'created_date');
        $values = array(
            $db->quote($agentid),
            $db->quote($invoiceId),
            $db->quote($status),
            $db->quote($amount),
            $db->quote(Factory::getDate()->toSql())
        );
        $query
            ->insert($db->quoteName('#__zonenationspay_btcpay_invoices'))
            ->columns($db->quoteName($columns))
            ->values(implode(',', $values));
        $db->setQuery($query);
        try {
            $result = $db->execute();
            if ($result) {
                return true;
            }
        } catch (Exception $e) {
            Factory::getApplication()->enqueueMessage('Error adding new BTCPay invoice: ' . $e->getMessage(), 'error');
        }
        return false;
    }

    public function updateInvoiceStatus($invoiceId, $newStatus)
    {
        $db = $this->db;
        $query = $db->getQuery(true);
        $fields = array(
            $db->quoteName('status') . ' = ' . $db->quote($newStatus),
            $db->quoteName('updated_date') . ' = ' . $db->quote(Factory::getDate()->toSql())
        );
        $conditions = array(
            $db->quoteName('invoiceid') . ' = ' . $db->quote($invoiceId)
        );
        $query->update($db->quoteName('#__zonenationspay_btcpay_invoices'))
            ->set($fields)
            ->where($conditions);
        $db->setQuery($query);
        try {
            return $db->execute();
        } catch (Exception $e) {
            Factory::getApplication()->enqueueMessage('Error updating invoice status: ' . $e->getMessage(), 'error');
            return false;
        }
    }

    public function getInvoiceDetails($invoiceId)
    {
        $db = $this->db;
        $query = $db->getQuery(true);
        $query->select('*')
            ->from($db->quoteName('#__zonenationspay_btcpay_invoices'))
            ->where($db->quoteName('invoiceid') . ' = ' . $db->quote($invoiceId));
        $db->setQuery($query);
        try {
            return $db->loadObject();
        } catch (Exception $e) {
            Factory::getApplication()->enqueueMessage('Error fetching invoice details: ' . $e->getMessage(), 'error');
            return null;
        }
    }

    protected function archivePullPayment($pullPaymentId)
    {
        $url = $this->params->get('btcpay_server_url') . "/api/v1/pull-payments/" . $pullPaymentId . "/archive";
        $response = $this->makeApiCall($url, 'POST');
        return $response && isset($response['id']);
    }

    protected function addNewBtcpayPullPayment($agentid, $pullPaymentId, $payoutId, $status, $fiatAmount, $localAmount)
    {
        $db = $this->db;
        $query = $db->getQuery(true);
        $columns = array('agentid', 'pullpaymentid', 'payoutid', 'status', 'fiat_amount', 'local_amount', 'created_date');
        $values = array(
            $db->quote($agentid),
            $db->quote($pullPaymentId),
            $db->quote($payoutId),
            $db->quote($status),
            $db->quote($fiatAmount),
            $db->quote($localAmount),
            $db->quote(Factory::getDate()->toSql())
        );
        $query
            ->insert($db->quoteName('#__zonenationspay_btcpay_pullpayments'))
            ->columns($db->quoteName($columns))
            ->values(implode(',', $values));
        $db->setQuery($query);
        try {
            $result = $db->execute();
            if ($result) {
                return true;
            }
        } catch (Exception $e) {
            Factory::getApplication()->enqueueMessage('Error adding new BTCPay pull payment: ' . $e->getMessage(), 'error');
        }
        return false;
    }

    protected function makeUrl($serverURI, $port)
    {
        $url = parse_url($serverURI);
        if (!isset($url['scheme'])) {
            $url['scheme'] = 'http';
        }
        if (!isset($url['host'])) {
            $url['host'] = $serverURI;
        }
        $url['port'] = $port;
        return array(
            'host' => $url['host'],
            'url' => $url['scheme'] . '://' . $url['host'] . ':' . $url['port']
        );
    }

    public static function formatAmount($amount, $currency)
    {
        $formatter = new NumberFormatter(Factory::getLanguage()->getTag(), NumberFormatter::CURRENCY);
        return $formatter->formatCurrency($amount, $currency);
    }

    public static function validateAddress($address, $crypto)
    {
        switch (strtoupper($crypto)) {
            case 'BTC':
                return (bool) preg_match('/^[13][a-km-zA-HJ-NP-Z1-9]{25,34}$/', $address);
            case 'ETH':
                return (bool) preg_match('/^0x[a-fA-F0-9]{40}$/', $address);
            // Add more cryptocurrency validations as needed
            default:
                return false;
        }
    }

    private function logError($message)
    {
        $logFile = JPATH_ROOT . '/logs/zonenationspay_btcpay_errors.php';
        $date = Factory::getDate();
        $logMessage = $date->format('Y-m-d H:i:s') . ' - ' . $message . "\n";
        file_put_contents($logFile, $logMessage, FILE_APPEND);
    }
}

