<?php
defined('_JEXEC') or die;

use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Stripe\Stripe;
use Stripe\PaymentIntent;
use Stripe\Exception\ApiErrorException;

require_once JPATH_LIBRARIES . '/vendor/autoload.php';
class ZonenationspayModelPayment extends BaseDatabaseModel
{
    private $stripeSecretKey;
    private $webhookSecret;

    public function __construct($config = array())
    {
        parent::__construct($config);
        $params = ComponentHelper::getParams('com_zonenationspay');
        $this->stripeSecretKey = $params->get('stripe_live_mode') ? $params->get('stripe_live_secret_key') : $params->get('stripe_test_secret_key');
        $this->webhookSecret = $params->get('stripe_webhook_secret');
        Stripe::setApiKey($this->stripeSecretKey);
    }

    public function processStripePayment($data)
    {
        try {
            if (isset($data['paymentMethodId'])) {
                $intent = PaymentIntent::create([
                    'amount' => $data['amount'] * 100,
                    'currency' => $data['currency'],
                    'payment_method' => $data['paymentMethodId'],
                    'confirmation_method' => 'manual',
                    'confirm' => true,
                ]);
            } elseif (isset($data['payment_intent_id'])) {
                $intent = PaymentIntent::retrieve($data['payment_intent_id']);
                $intent->confirm();
            } else {
                return ['error' => 'Invalid request'];
            }

            if ($intent->status == 'requires_action' &&
                $intent->next_action->type == 'use_stripe_sdk') {
                return [
                    'requires_action' => true,
                    'payment_intent_client_secret' => $intent->client_secret
                ];
            } elseif ($intent->status == 'succeeded') {
                $this->saveTransaction($intent->id, $intent->amount / 100, $intent->currency);
                return [
                    'success' => true,
                    'success_url' => JRoute::_('index.php?option=com_zonenationspay&view=payment&layout=success')
                ];
            } else {
                return ['error' => 'Invalid PaymentIntent status'];
            }
        } catch (ApiErrorException $e) {
            return ['error' => $e->getMessage()];
        }
    }

    public function handleWebhook()
    {
        $payload = @file_get_contents('php://input');
        $sigHeader = $_SERVER['HTTP_STRIPE_SIGNATURE'];
        $event = null;
        try {
            $event = \Stripe\Webhook::constructEvent(
                $payload, $sigHeader, $this->webhookSecret
            );
        } catch(\UnexpectedValueException $e) {
            http_response_code(400);
            exit();
        } catch(\Stripe\Exception\SignatureVerificationException $e) {
            http_response_code(400);
            exit();
        }

        switch ($event->type) {
            case 'payment_intent.succeeded':
                $paymentIntent = $event->data->object;
                $this->handleSuccessfulPayment($paymentIntent);
                break;
            case 'payment_intent.payment_failed':
                $paymentIntent = $event->data->object;
                $this->handleFailedPayment($paymentIntent);
                break;
            default:
                http_response_code(400);
                exit();
        }

        http_response_code(200);
    }

    private function handleSuccessfulPayment($paymentIntent)
    {
        // Update the order status in your database
        $this->updateTransactionStatus($paymentIntent->id, 'completed');
        // You might want to trigger any post-payment processes here
    }

    private function handleFailedPayment($paymentIntent)
    {
        // Update the order status in your database
        $this->updateTransactionStatus($paymentIntent->id, 'failed');
    }

    private function saveTransaction($transactionId, $amount, $currency)
    {
        $db = $this->getDbo();
        $query = $db->getQuery(true);

        $columns = ['transaction_id', 'amount', 'currency', 'status', 'created_at'];
        $values = [
            $db->quote($transactionId),
            $db->quote($amount),
            $db->quote($currency),
            $db->quote('pending'),
            $db->quote(Factory::getDate()->toSql())
        ];

        $query
            ->insert($db->quoteName('#__zonenationspay_transactions'))
            ->columns($db->quoteName($columns))
            ->values(implode(',', $values));

        $db->setQuery($query);
        $db->execute();
    }

    private function updateTransactionStatus($transactionId, $status)
    {
        $db = $this->getDbo();
        $query = $db->getQuery(true);

        $query
            ->update($db->quoteName('#__zonenationspay_transactions'))
            ->set($db->quoteName('status') . ' = ' . $db->quote($status))
            ->where($db->quoteName('transaction_id') . ' = ' . $db->quote($transactionId));

        $db->setQuery($query);
        $db->execute();
    }
}
